﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Dynamic;
using System.Linq;
using System.Threading.Tasks;
using FinecarRentCar.DAL;
using FinecarRentCar.Data;
using FinecarRentCar.Helpers;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Syncfusion.EJ2.Base;
using Syncfusion.EJ2.Navigations;
using Newtonsoft.Json;

namespace FinecarRentCar.Controllers
{
    public class CarsController : Controller
    {
        private ICarRepository _carRepository = new CarRepository(new FinecarDbContext());
        private FinecarDbContext db = new FinecarDbContext();
        public IActionResult Index()
        {

            var carsList = db.Cars.ToList();
            ViewBag.carDataSource = carsList;
            return View();
        }
        public IActionResult CarDataSource([FromBody]DataManagerRequest dm)
        {
            IEnumerable DataSource = db.Cars.Select(car => new Models.AllCarsViewModel()
            {
                CarId = car.ID,
                TagNumber = car.TagNumber,
                Tachometer = car.Tachometer,
                CarFullName = car.Make + " " + car.Model + " " + car.Performance,
                Make = car.Make,
                Model = car.Model,
                Performance = car.Performance,
                ImageFileName = car.ImageFileName,
                CategoryName = _carRepository.GetCategoryNameByCarId(car.ID),
                CarYear = car.CarYear,
                FullInfo = car.FullInfo,
                Fuel = car.Fuel,
                Transmission = car.Transmission,
                isAvailable = car.isAvailable,
                MP3Player = car.MP3Player,
                DVDPlayer = car.DVDPlayer
            }).ToList();
            DataOperations operation = new DataOperations();
            if (dm.Search != null && dm.Search.Count > 0)
            {
                DataSource = operation.PerformSearching(DataSource, dm.Search);
            }
            if (dm.Sorted != null && dm.Sorted.Count > 0) //Sorting
            {
                DataSource = operation.PerformSorting(DataSource, dm.Sorted);
            }
            if (dm.Where != null && dm.Where.Count > 0) //Filtering
            {
                DataSource = operation.PerformFiltering(DataSource, dm.Where, dm.Where[0].Operator);
            }
            int count = DataSource.Cast<object>().Count();
            if (dm.Skip != 0)
            {
                DataSource = operation.PerformSkip(DataSource, dm.Skip);   //Paging
            }
            if (dm.Take != 0)
            {
                DataSource = operation.PerformTake(DataSource, dm.Take);
            }
            return dm.RequiresCounts ? Json(new { result = DataSource, count = count }) : Json(DataSource);
        }

        public ActionResult AddPartial([FromBody] CRUDModel<Models.AllCarsViewModel> value)
        {
            ViewBag.fuelDataSource = new string[] { "Benzín", "Nafta", "LPG", "CNG" };
            ViewBag.transmissionDataSource = new string[] { "Manuální", "Automatická DSG", "Automatická CVT", "Mechanická automatická", "Hydrodynamická s uzamykáním" };

            var categorySource = db.Categories.ToList();
            ViewBag.categoryDataSource = categorySource;

            return PartialView("_DialogAddPartial");
        }

        public IActionResult EditPartial([FromBody] CRUDModel<Models.AllCarsViewModel> value)
        {
            ViewBag.fuelDataSource = new string[] { "Benzin", "Nafta", "LPG", "CNG" };
            ViewBag.transmissionDataSource = new string[] { "Manuální", "Automatická DSG", "Automatická CVT", "Mechanická automatická", "Hydrodynamická s uzamykáním" };

            var categorySource = db.Categories.ToList();
            ViewBag.categoryDataSource = categorySource;

            return PartialView("_DialogEditPartial", value.Value);
        }

        #region CRUD
        public JsonResult Insert([FromBody]CRUDModel<Models.AllCarsViewModel> value)
        {
            var entity = new Models.Entities.Car
            {
                TagNumber = value.Value.TagNumber,
                Make = value.Value.Make,
                Model = value.Value.Model,
                Transmission = value.Value.Transmission,
                Fuel = value.Value.Fuel,
                Performance = value.Value.Performance,
                CarYear = value.Value.CarYear,
                isAvailable = value.Value.isAvailable,
                MP3Player = value.Value.MP3Player,
                DVDPlayer = value.Value.DVDPlayer,
                AirConditioner = value.Value.AirConditioner,
                ABS = value.Value.ABS,
                ASR = value.Value.ASR,
                Navigation = value.Value.Navigation,
                FullInfo = value.Value.FullInfo,
                Color = value.Value.Color,
                Seats = value.Value.Seats,
                Consumption = value.Value.Consumption,
                Airbags = value.Value.Airbags,
                Door = value.Value.Door,
                Shifts = value.Value.Shifts,
                ImageFileName = value.Value.ImageFileName,
                UrlSeo = StringHelper.URLFriendly(value.Value.Make + " " + value.Value.Model + " " + value.Value.Performance, 150),
                Meta = value.Value.Meta,
                Tachometer = value.Value.Tachometer,
                Inserted = DateTime.Now,
                Updated = DateTime.Now
            };

            db.Cars.ToList().Insert(0, entity);

            value.Value.CarId = entity.ID;
            int lastCarId = entity.ID;

            int categoryID = _carRepository.GetCategoryByName(value.Value.CategoryName);

            var carToCategory = new Models.Entities.CarCategories
            {
                CarID = lastCarId,
                CategoryID = categoryID
            };
            db.CarCategories.Add(carToCategory);
            db.SaveChanges();


            return new JsonResult(value.Value);
        }
        #endregion

    }
}